package tw.com.lig.sdk.examples

import android.app.Activity
import android.app.ActivityManager
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.util.Log
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.google.ar.core.TrackingState
import com.google.ar.sceneform.AnchorNode
import com.google.ar.sceneform.math.Matrix
import com.google.ar.sceneform.math.Quaternion
import com.google.ar.sceneform.math.Vector3
import com.google.ar.sceneform.rendering.Color
import com.google.ar.sceneform.rendering.MaterialFactory
import com.google.ar.sceneform.rendering.ModelRenderable
import com.google.ar.sceneform.rendering.ShapeFactory
import com.google.ar.sceneform.ux.TransformableNode

class ARSceneActivity : AppCompatActivity() {
    private var mArFragment: ArFragment? = null
    lateinit var transform: FloatArray
    private var cubeRenderable: ModelRenderable? = null
    private val anchorNode = AnchorNode()
    private val axesNode = AnchorNode()

    private val handler: Handler = object : Handler() {
        override fun handleMessage(msg: Message) {
            if (msg.what == SHOW_OBJECT) {
                getPoseFromMatrix(transform = transform)
                Log.d("steven", "show model ........")
            } else if (msg.what == INIT_AR) {
                postDelayed(runnable, 300)
            }
            super.handleMessage(msg)
        }
    }

    var runnable = Runnable {
        mArFragment?.arSceneView?.arFrame?.let {
            if(it.camera.trackingState == TrackingState.TRACKING){
                Log.d("steven", "trackingState ok  ........")
                handler.sendEmptyMessage(SHOW_OBJECT)
            }else{
                handler.sendEmptyMessage(INIT_AR)
            }
        }
    }

    private fun checkIsSupportedDeviceOrFinish(activity: Activity): Boolean {
        val openGlVersionString = (activity.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager)
            .deviceConfigurationInfo
            .glEsVersion
        if (openGlVersionString.toDouble() < MIN_OPEN_GL_VERSION) {
            Log.i(TAG, "Sceneform requires OpenGL ES 3.0 later")
            Toast.makeText(activity, "Sceneform requires OpenGL ES 3.0 or later", Toast.LENGTH_LONG)
                .show()
            activity.finish()
            return false
        }
        return true
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (!checkIsSupportedDeviceOrFinish(this)) {
            return
        }
        setContentView(R.layout.activity_show_object)

        mArFragment = supportFragmentManager.findFragmentById(R.id.ux_fragment) as ArFragment?
        mArFragment!!.arSceneView.planeRenderer.isVisible = false
        handler.postDelayed(runnable, 300)

        transform = intent.getFloatArrayExtra("transform")!!
    }

    fun getPoseFromMatrix(transform: FloatArray) {
        val matrix = Matrix(transform)
        val position = Vector3()
        matrix.decomposeTranslation(position)
        val quaternion = Quaternion()
        matrix.extractQuaternion(quaternion)

        MaterialFactory.makeOpaqueWithColor(this, Color(191f, 0f, 0f))
            .thenAccept {
                val center = Vector3(0f, 0f, 0f)
                cubeRenderable = ShapeFactory.makeCube(Vector3(0.1f, 0.1f, 0.1f), center, it)
                val axes = TransformableNode(mArFragment!!.transformationSystem)
                axes.renderable = cubeRenderable
                axes.setParent(anchorNode)
                anchorNode.worldRotation = quaternion
                anchorNode.worldPosition = position.scaled(0.001f)

                anchorNode.setParent(mArFragment!!.arSceneView.scene)
            }

        createAxis("x", position, quaternion)
        createAxis("y", position, quaternion)
        createAxis("z", position, quaternion)

    }

    private fun createAxis(whichAxis: String, position: Vector3, quaternion: Quaternion) {
        var color: Color? = null
        when (whichAxis) {
            "x" -> {
                color = Color(191f, 0f, 0f)
            }
            "y" -> {
                color = Color(0f, 127f, 63f)
            }
            "z" -> {
                color = Color(0f, 0f, 191f)
            }
        }
        MaterialFactory.makeOpaqueWithColor(this, color)
            .thenAccept {
                val center: Vector3?
                var xAxisRenderable: ModelRenderable? = null
                when (whichAxis) {
                    "x" -> {
                        center = Vector3(0.2f, 0f, 0f)
                        xAxisRenderable = ShapeFactory.makeCube(Vector3(0.4f, AXIS_WIDTH, AXIS_WIDTH), center, it)
                    }
                    "y" -> {
                        center = Vector3(0f, 0.2f, 0f)
                        xAxisRenderable = ShapeFactory.makeCube(Vector3(AXIS_WIDTH, 0.4f, AXIS_WIDTH), center, it)
                    }
                    "z" -> {
                        center = Vector3(0f, 0f, 0.2f)
                        xAxisRenderable = ShapeFactory.makeCube(Vector3(AXIS_WIDTH, AXIS_WIDTH, 0.4f), center, it)
                    }
                }
                val axes = TransformableNode(mArFragment!!.transformationSystem)
                axes.renderable = xAxisRenderable
                axes.setParent(axesNode)
                axesNode.worldPosition = Vector3(position.x + 100f, position.y, position.z).scaled(0.001f)
                axesNode.setParent(mArFragment!!.arSceneView.scene)
                if (whichAxis == "z") {
                    axesNode.worldRotation = quaternion
                }
            }
    }

    companion object {
        const val INIT_AR = 0x0001
        const val SHOW_OBJECT = 0x0011
        const val MIN_OPEN_GL_VERSION = 3.0
        const val TAG = "ARSceneActivity"
        const val AXIS_WIDTH = 0.002f
    }
}