package tw.com.lig.sdk.examples

import android.util.Log
import android.widget.Toast
import com.google.ar.core.Config
import com.google.ar.core.Session
import com.google.ar.core.exceptions.*
import com.google.ar.sceneform.FrameTime
import com.google.ar.sceneform.ux.BaseArFragment

class ArFragment : BaseArFragment() {
    override fun isArRequired(): Boolean {
        return true
    }

    override fun getAdditionalPermissions(): Array<String?> {
        return arrayOfNulls(0)
    }

    override fun handleSessionException(sessionException: UnavailableException) {
        val message: String = when (sessionException) {
            is UnavailableArcoreNotInstalledException -> {
                "Please install ARCore"
            }
            is UnavailableApkTooOldException -> {
                "Please update ARCore"
            }
            is UnavailableSdkTooOldException -> {
                "Please update this app"
            }
            is UnavailableDeviceNotCompatibleException -> {
                "This device does not support AR"
            }
            else -> {
                "Failed to create AR session"
            }
        }
        Log.e(TAG, "Error: $message", sessionException)
        Toast.makeText(requireActivity(), message, Toast.LENGTH_LONG).show()
    }

    override fun getSessionConfiguration(session: Session): Config {
        return Config(session)
    }

    override fun getSessionFeatures(): Set<Session.Feature> {
        return emptySet()
    }

    override fun onUpdate(frameTime: FrameTime) {
        super.onUpdate(frameTime)
        planeDiscoveryController.hide()
    }

    override fun onResume() {
        super.onResume()
        planeDiscoveryController.hide()
    }

    companion object {
        private const val TAG = "StandardArFragment"
    }
}